﻿using Microsoft.AspNetCore.SignalR.Client;
using Microsoft.Extensions.Logging;
using System;
using System.Threading.Tasks;

namespace demo
{
    public enum RepeatedOrderArticleState
    {
        SCHEDULED = 0,
        READYFORINPUT = 1,
        READYFORPICKUP = 2,
        PICKEDUP = 3,
        CANCELED = 4,
        OVERDUE = 5,
        CANCELREQUESTED = 6,
        DISPENSEREQUESTED = 7
    }

    public class ArticleStatusChangedResponse
    {
        public string ArticleId { get; set; }
        public int RequiredQuantity { get; set; }
        public DateTime DueDate { get; set; }
        public DateTime? OrderArticleStateChangedDate { get; set; }
        public RepeatedOrderArticleState OrderArticleState { get; set; }
    }

    class Program
    {
        private static HubConnection _hubConnection;
        
        static async Task Main(string[] args)
        {
            try
            {
                if (args.Length != 2)
                {
                    Console.WriteLine("usage: demo.exe url-to-api-host apikey");
                    return;
                }

                string _webApiHubUrl = args[0].Trim();
                string apiKey = args[1].Trim();

                _hubConnection = new HubConnectionBuilder()
                    .WithUrl($"{_webApiHubUrl}/rt-api/orderstate", options => {
                        options.AccessTokenProvider = () => Task.FromResult("apikey-" + apiKey);
                    })
                    .ConfigureLogging(logging => {
                        // Log to the Console (do NOT do this in production)
                        logging.AddConsole();

                        // This will set ALL logging to Debug level
                        logging.SetMinimumLevel(LogLevel.Debug);
                    })
                    .Build();                

                _hubConnection.On<string, string, string, DateTime?, string>("OrderStateChanged", (orderNumber, pickupCode, orderState, orderStateChangedDate, extraInfo) =>
                {
                    Console.WriteLine($"Order '{orderNumber}' with pickupCode '{pickupCode}' and date '{orderStateChangedDate}' is now in state {orderState}.");
                    if (!string.IsNullOrEmpty(extraInfo))
                        Console.WriteLine($"Extra info: {extraInfo}");
                });

                _hubConnection.On<string, string, ArticleStatusChangedResponse, string>("OrderArticleStateChanged", (orderNumber, pickupCode, articleStatusChangedResponse, extraInfo) =>
                {
                    Console.WriteLine($"\tArticle of order with orderNumber = '{orderNumber}', pickupCode = '{pickupCode}' and: \n {FormatArticleIterationInfo(articleStatusChangedResponse)} \n\t\n");
                });

                // start the connection
                await _hubConnection.StartAsync();

                Console.WriteLine("Connected. Press enter to disconnect...");
                Console.ReadLine();

                Console.WriteLine("disconnecting...");
                //stop the connection
                await _hubConnection.StopAsync();

            }
            catch (Exception ex)
            {
                Console.WriteLine($"An error has occured: {ex}");
            }
        }

        private static string FormatArticleIterationInfo(ArticleStatusChangedResponse articleStatusChangedResponse)
        {
            return $"\t\tArticleId = {articleStatusChangedResponse.ArticleId}" +
            $"\n\t\tRequired quantity = {articleStatusChangedResponse.RequiredQuantity}" +
            $"\n\t\tDue date = {articleStatusChangedResponse.DueDate}" +
            $"\n\t\tChanged date = {articleStatusChangedResponse.OrderArticleStateChangedDate}" +
            $"\n\t\tState = {articleStatusChangedResponse.OrderArticleState}";
        }
    }
}
